@@ -170,10 +170,8 @@ set_native_charset (const char *newset)
          different one for console input.  Not sure how to cope with
          that.  If the console Code page is not known we fall back to
          the system code page.  */
-#ifndef HAVE_W32CE_SYSTEM
       cpno = GetConsoleOutputCP ();
       if (!cpno)
-#endif
         cpno = GetACP ();
       sprintf (codepage, "CP%u", cpno );
       /* Resolve alias.  We use a long string string and not the usual
@@ -301,7 +299,7 @@ set_native_charset (const char *newset)
 }
 
 const char *
-get_native_charset ()
+get_native_charset (void)
 {
   return active_charset_name;
 }
@@ -806,10 +804,8 @@ get_w32_codepage (void)
 
   if (!cp)
     {
-#ifndef HAVE_W32CE_SYSTEM
       cp = GetConsoleOutputCP ();
       if (!cp)
-#endif
         cp = GetACP ();
     }
   return cp;
